#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#define MAX(a,b) ((a) > (b) ? (a) : (b))
#define MIN(a,b) ((a) > (b) ? (b) : (a))

#define eps 1.0e-7
#define epd 1.0e-5
#define EPS 2.0e-6
#define MAXM 100
#define MAXLINE 1024
#define EPSS 6.e-8
#define MAXIT 100
#define tiny 1.0e-20

void derror(char error_text[]){
  void exit();
  fprintf(stderr,"Run-time error...\n");
  fprintf(stderr,"%s\n",error_text);
  fprintf(stderr,"Exiting to system\n");
  exit(1);
}

float *vector(int n_min,int n_max){
  float *v;
  v=(float *)malloc((unsigned) (n_max-n_min+1)*sizeof(float));
  if (!v) derror("allocation failure in vector()");
  return v-n_min;
}

int *ivector(int n_min, int n_max){
  int *v;
  v=(int *)malloc((unsigned) (n_max-n_min+1)*sizeof(int));
  if (!v) derror("allocation failure in ivector()");
  return v-n_min;
}

float **matrix(int nr_min,int nr_max,int nc_min,int nc_max){
  int i;
  float **m;
  m=(float **) malloc((unsigned) (nr_max-nr_min+1)*sizeof(float*));
  if (!m) derror("allocation failure 1 in matrix()");
  m -= nr_min;
  for(i=nr_min;i<=nr_max;i++) {
    m[i]=(float *) malloc((unsigned) (nc_max-nc_min+1)*sizeof(float));
    if (!m[i]) derror("allocation failure 2 in matrix()");
    m[i] -= nc_min;
  }
  return m;
}

void free_vector(float *v,int n_min,int n_max){
  free((char*) (v+n_min));
}

void free_ivector(int *v, int n_min,int n_max){
  free((char*) (v+n_min));
}

void free_matrix(float **m, int nr_min, int nr_max, 
    int nc_min, int nc_max){
  int i;
  for(i=nr_max;i>=nr_min;i--) free((char*) (m[i]+nc_min));
  free((char*) (m+nr_min));
}

typedef struct FCOMPLEX {float r,i;} fcomplex;

fcomplex Cadd(fcomplex a, fcomplex b){
  fcomplex c;

  c.r=a.r+b.r;
  c.i=a.i+b.i;
  return c;
}

fcomplex Csub(fcomplex a, fcomplex b){
  fcomplex c;

  c.r=a.r-b.r;
  c.i=a.i-b.i;
  return c;
}

fcomplex Cmul(fcomplex a, fcomplex b){
  fcomplex c;

  c.r=a.r*b.r-a.i*b.i;
  c.i=a.i*b.r+a.r*b.i;
  return c;
}

fcomplex Complex(float re, float im){
  fcomplex c;

  c.r=re;
  c.i=im;
  return c;
}

fcomplex Conjg(fcomplex z){
  fcomplex c;

  c.r=z.r;
  c.i = -z.i;
  return c;
}

fcomplex Cdiv(fcomplex a, fcomplex b){
  fcomplex c;

  float r,den;
  if (fabs(b.r) >= fabs(b.i)) {
    r=b.i/b.r;
    den=b.r+r*b.i;
    c.r=(a.r+r*a.i)/den;
    c.i=(a.i-r*a.r)/den;
  }
  else {
    r=b.r/b.i;
    den=b.i+r*b.r;
    c.r=(a.r*r+a.i)/den;
    c.i=(a.i*r-a.r)/den;
  }
  return c;
}

float Cabs(fcomplex z){
  float x,y,ans,temp;

  x=fabs(z.r);
  y=fabs(z.i);
  if (x == 0.0)
    ans=y;
  else if (y == 0.0)
    ans=x;
  else if (x > y) {
    temp=y/x;
    ans=x*sqrt(1.0+temp*temp);
  }
  else {
   temp=x/y;
   ans=y*sqrt(1.0+temp*temp);
  }
  return ans;
}

fcomplex Csqrt(fcomplex z){
  fcomplex c;
  float x,y,w,r;

  if ((z.r == 0.0) && (z.i == 0.0)) {
    c.r=0.0;
    c.i=0.0;
    return c;
  }
  else {
    x=fabs(z.r);
    y=fabs(z.i);
    if (x >= y) {
      r=y/x;
      w=sqrt(x)*sqrt(0.5*(1.0+sqrt(1.0+r*r)));
    }
    else {
      r=x/y;
      w=sqrt(y)*sqrt(0.5*(r+sqrt(1.0+r*r)));
    }
    if (z.r >= 0.0) {
      c.r=w;
      c.i=z.i/(2.0*w);
    }
    else {
      c.i=(z.i >= 0) ? w : -w;
      c.r=z.i/(2.0*c.i);
    }
    return c;
  }
}

fcomplex RCmul(float x,fcomplex a){
  fcomplex c;

  c.r=x*a.r;
  c.i=x*a.i;
  return c;
}


void piksrt(int n,float arr[]){
  int i,j;
  float a;

  for (j=2;j<=n;j++) {
    a=arr[j];
    i=j-1;
    while (i > 0 && arr[i] > a) {
      arr[i+1]=arr[i];
      i--;
    }
    arr[i+1]=a;
  }
}

void laguer(fcomplex a[], int m, fcomplex *x, float ep, int plsh){
  int j,iter;
  float err,dxold,cdx,abx;
  fcomplex sq,h,gp,gm,g2,g,b,d,dx,f,x1;

  dxold=Cabs(*x);
  for (iter=1;iter<=MAXIT;iter++) {
    b=a[m];
    err=Cabs(b);
    d=Complex(0.0,0.0);
    f=Complex(0.0,0.0);
    abx=Cabs(*x);
    for (j=m-1;j>=0;j--) {
      f=Cadd(Cmul(*x,f),d);
      d=Cadd(Cmul(*x,d),b);
      b=Cadd(Cmul(*x,b),a[j]);
      err=Cabs(b)+abx*err;
    }
    err *= EPSS;
    if (Cabs(b) <= err) return;
    g=Cdiv(d,b);
    g2=Cmul(g,g);
    h=Csub(g2,RCmul(2.0,Cdiv(f,b)));
    sq=Csqrt(RCmul((float) (m-1),Csub(RCmul((float) m,h),g2)));
    gp=Cadd(g,sq);
    gm=Csub(g,sq);
    if (Cabs(gp) < Cabs(gm))gp=gm;
    dx=Cdiv(Complex((float) m,0.0),gp);
    x1=Csub(*x,dx);
    if (x->r == x1.r && x->i == x1.i) return;
    *x=x1;
    cdx=Cabs(dx);
    if (iter > 6 && cdx >= dxold) return;
    dxold=cdx;
    if (!plsh)
      if (cdx <= ep*Cabs(*x)) return;
  }
  derror("Too many iterations in routine LAGUER");
}

void RRoots(float a[], int m, float rroots[]){
  int jj,j,i;
  fcomplex x,b,c,ad[MAXM],roots[MAXM],iroot;
  float xr;

  for (j=0;j<=m;j++) ad[j]=Complex(a[j],0.0);
  for (j=m;j>=1;j--) {
    x=Complex(0.0,0.0);
    laguer(ad,j,&x,EPS,0);
    if (fabs(x.i) <= (2.0*EPS*fabs(x.r))) x.i=0.0;
    roots[j]=x;
    b=ad[j];
    for (jj=j-1;jj>=0;jj--) {
      c=ad[jj];
      ad[jj]=b;
      b=Cadd(Cmul(x,b),c);
    }
  }
  for (j=0;j<=m;j++) ad[j]=Complex(a[j],0.0);
  for (j=1;j<=m;j++)
    laguer(ad,m,&roots[j],EPS,1);

  for (j=1;j<=m;j++)
    rroots[j] = roots[j].r;
  piksrt(m,rroots);
}

void four1(float dat[],int nn,int isign){
  int n,mmax,m,j,istep,i;
  double wtemp,wr,wpr,wpi,wi,theta;
  float tempr,tempi,dum;

  n=nn << 1;
  j=1;
  for (i=1;i<n;i+=2) {
    if (j > i) {
      dum = dat[j];
      dat[j] = dat[i];
      dat[i] = dum;
      dum = dat[j+1];
      dat[j+1] = dat[i+1];
      dat[i+1] = dum;
    }
    m=n >> 1;
    while (m >= 2 && j > m) {
      j -= m;
      m >>= 1;
    }
    j += m;
  }
  mmax=2;
  while (n > mmax) {
    istep=2*mmax;
    theta=6.28318530717959/(isign*mmax);
    wtemp=sin(0.5*theta);
    wpr = -2.0*wtemp*wtemp;
    wpi=sin(theta);
    wr=1.0;
    wi=0.0;
    for (m=1; m<mmax; m+=2) {
      for (i=m; i<=n; i+=istep) {
        j=i+mmax;
        tempr=wr*dat[j]-wi*dat[j+1];
        tempi=wr*dat[j+1]+wi*dat[j];
        dat[j]=dat[i]-tempr;
        dat[j+1]=dat[i+1]-tempi;
        dat[i] += tempr;
        dat[i+1] += tempi;
      }
      wr=(wtemp=wr)*wpr-wi*wpi+wr;
      wi=wi*wpr+wtemp*wpi+wi;
    }
    mmax=istep;
  }
}

int nextpow2(int i){
/*
 *  Computes the next largest power of 2 of i
 */
  int j=1;

  if ( i < 0 ) derror("Input negative to nextpow2");
  while ( i >= j ) j = 2*j;
  return j;
}

float polyval (float *alpha, int degree, float ordinate){
/*
 *  Evaluate a polynomial at ordinate.  Use Horners method.
 *  alpha is an array of coefficients of the polynomial
 *  arranged in ascending order.
 *  
 */

  int i;
  float poly;

  if ( degree < 0 )
    derror("Polynomial degree must be positive");

/*
 * evaluate
 */
  poly = alpha[degree];
  for (i=degree-1; i>=0; i--)
     poly = (poly*ordinate) + alpha[i];

  return poly;
}

float interp1 (float *omega, float *data, int num_data,
	       float ordinate){
/*
 *  Compute the piecewise linear interpolant of data at
 *  the point ordinate.
 *
 *  Returns the interpolated value.
 */

  int i1,i2,i3;
  float theta;

  if ( ordinate < omega[0] )
    derror("Interpolation point less than first node");
  if ( ordinate > omega[num_data-1] )
    derror("Interpolation point greater than last node");
    
/*
 * determine where to interpolate 
 */
  i1 = 0;  i2 = num_data-1;
  while (i2-i1>1){
    i3 = (i2+i1+1)/2;
    if ( ordinate >= omega[i3] ) i1 = i3;
    else i2 = i3;
  }

/*
 * interpolate
 */
  theta = (omega[i2]-ordinate)/(omega[i2]-omega[i1]);
  return (theta*data[i1]) + ((1.-theta)*data[i2]);
}

void ludcmp(float **a,int n,int *indx){
  int i,j,k,p,bigi,itemp;
  int idat,jdat;
  float big,sum,temp;
  void derror();

/*
 *  load the index vector
 */
  for (k=1;k<=n;k++) indx[k]=k;

  for (k=1;k<=n;k++) {

/*
 *  determine row to swap
 */
    big = fabs(a[k][k]);
    bigi = k;
    for (i=k+1;i<=n;i++) {
      if ( MAX(big,(temp=fabs(a[i][k]))) != big ){
        big = temp;
        bigi = i;
      }
    }
   
/*
 *  outta here if its singular
 */
    if (big < tiny) derror("Matrix is singular...");
 
/*
 *  swap them.
 */
    if (k != bigi) {
      itemp = indx[k];
      indx[k] = bigi;
      indx[bigi] = itemp;
      for (j=1;j<=n;j++){
        temp =a[bigi][j];
        a[bigi][j]=a[k][j];
        a[k][j]=temp;
      }
    }

/*
 *  u_kj
 */
    for (j=k;j<=n;j++) {
      sum=a[k][j];
      for (p=1;p<=k-1;p++) sum -= a[k][p]*a[p][j];
      a[k][j]=sum;
    }

/*
 *  l_ik
 */
    for (i=k+1;i<=n;i++) {
      sum=a[i][k];
      for (p=1; p<=k-1; p++) sum -= a[i][p]*a[p][k];
      a[i][k]=sum/a[k][k];
    }

  }
}

void lubksb(float **a, int n, int *indx, float b[]){

  int i,j,k;
  float sum;
  float *vv,*vector();
  void free_vector();

  vv = vector(1,n);
  for (k=1;k<=n;k++) vv[k] = b[indx[k]];

/*
 *  forward substitute (Ly=b)
 */
  b[1] = vv[1];
  for (i=2;i<=n;i++) {
    sum = vv[i];
    for (j=1;j<=i-1;j++) sum -= a[i][j]*b[j];
    b[i]=sum;
  }
  for (k=1;k<=n;k++) vv[k] = b[k];

/*
 *  backward substitute (Ux=y)
 */
  for (i=n;i>=1;i--) {
    sum=b[i];
    for (j=i+1;j<=n;j++) sum -= a[i][j]*b[j];
    b[i]=sum/a[i][i];
  }

  free_vector(vv,1,n);
}

void hilbert (float *idata, int num_data, float *rdata){
/*
 *  Compute real data from imaginary data via the Hilbert
 *  transform
 *
 *  Returns the real data
 */

  float *data, junk;
  int num_d, j,k;

  num_d = 4*num_data + 4;
  data = vector(0,num_d);
  for (j=1; j<=num_d; j++) data[j]=0.0;

  for (j=1; j<=num_data; j++){
    data[2*j-1] = idata[j-1];
    data[2*j] = 0.0;
  }
  four1(data,2*num_data,1);

  k = num_data/2;
  for (j=2; j<=num_data; j++){
    data[2*j-1] = 2.0*data[2*j-1];
    data[2*j] = 2.0*data[2*j];
  }   
  for (j=num_data+2; j<=num_d; j++){
    data[j] = 0.0;
  }
  four1(data,2*num_data,-1);

  junk = 2*num_data;
  for (j=1; j<=num_data; j++){
    data[2*j-1] = data[2*j-1]/junk;
    data[2*j] = data[2*j]/junk;
  }

  for (j=1; j<=num_data; j++)
    rdata[j-1] = data[2*j];
}

void bnds (float *Omega, float *ieps, int num_data,
           float *aOmega, float *areps, int num_anchors,
           float *up_bound, float *lo_bound){
/*
 *  Compute the m-point finite frequency Kramers-Kronig
 *  relation.  See Milton, Eyre and Mantese.
 *
 *  On entry:
 *
 *    Omega    -- frequencies of imaginary part of eps
 *    ieps     -- imaginary part of the dielectric constant
 *    num_data -- number of data points
 *    aOmega   -- frequencies of the anchors
 *    areps    -- anchors, real part of the dielectric constant
 *    num_anchors -- number of anchors
 *
 *  On return:
 *
 *    lo_bound -- lower m-point bound at frequencies Omega
 *    up_bound -- upper m-point bound at frequencies Omega
 *
 */

   float **matrix(),*vector();
   int *ivector();
   void free_matrix(),free_vector();

   int j,jdat,jpol,k,kpol,m,n;    /* counters etc... */
   float junk;

   int num_z;                     /* rescale frequencies */
   float Om_max;
   float *omega,*zed;
   float *z,*iepsi;
   float step, z_minus, z_plus;
   float *kkrepsz,*kkreps;        /* K-K real variables */
   int azedi;                     /* anchor variables */
   float *azed,*aepkk,anchzc,anchec;

   float *fb1, bcheck;            /* one point bound variables */

   int npol[] = {1, 3, 3, 1};     /* m point bound variables */
   float z1[4],z2[4],z3[4],ppol[4];
   float lbchk, ubchk;
   int alpha, beta, fbm_ok, kflag, kstop;
   int *indx,isrt;
   float Q,*fQ;
   float **A,*b;
   float *anchzed,*ancheps;
   float *denp,*nump;
   float *pol,*zer;
   float *numer,*denom,*fbm;
   float *fb_lo, *fb_up;
   float *lace, *slace, lsrt, lmin;

   omega = vector(0,num_data);
   zed = vector(0,num_data);
   kkreps = vector(0,num_data);
   azed = vector(0,num_anchors);
   aepkk = vector(0,num_anchors);
   fb1 = vector(0,num_data);
   anchzed = vector(0,num_anchors);
   ancheps = vector(0,num_anchors);
   denp = vector(0,num_anchors);
   nump = vector(0,num_anchors);
   pol  = vector(0,num_anchors+3);
   zer  = vector(0,num_anchors);
   numer  = vector(0,num_data);
   denom  = vector(0,num_data);
   fbm    = vector(0,num_data);
   fb_lo  = vector(0,num_data);
   fb_up  = vector(0,num_data);
   lace  = vector(0,num_anchors);
   slace = vector(0,num_anchors);


/*
 *  check the frequencies
 */
   for (j=0; j<=num_data-2; j++)
     if ( Omega[j] >= Omega[j+1] )
       derror("The input frequencies must be ordered");

/*
 *  rescale the frequencies
 */
   Om_max = Omega[num_data-1];
   for (j=0; j<=num_data-1; j++) {
     omega[j] = Omega[j]/Om_max;
     zed[j] = omega[j]*omega[j];
   }
   z_minus = zed[0];
   z_plus = zed[num_data-1];

/*
 *  interpolate to regular spacing
 */
   num_z = nextpow2(num_data);
   z = vector(0,num_z);
   iepsi = vector(0,num_z);
   kkrepsz = vector(0,num_z);

   step = (z_plus-z_minus)/(junk=num_z-1);
   for (j=0; j<=num_z-2; j++){
     z[j] = zed[0] + ((junk=j)*step);
     iepsi[j] = interp1(zed,ieps,num_data,z[j]);
   }
   z[num_z-1] = zed[num_data-1];
   iepsi[num_z-1] = ieps[num_data-1];

/*
 *  Hilbert tranform in new variables to calculate real 
 *  eps, and interpolate back to original spacing
 */
   hilbert(iepsi,num_z,kkrepsz);
   for (j=0; j<=num_data-2; j++)
     kkreps[j] = interp1(z,kkrepsz,num_z,zed[j]);
   kkreps[num_data-1] = kkrepsz[num_z-1];

/*
 *  data anchors
 */
   for (j=0; j<=num_anchors-1; j++) {
     junk    = aOmega[j]/Om_max;
     azed[j]  = junk*junk;
     aepkk[j] = areps[j] - interp1(zed,kkreps,num_data,azed[j]-eps);
   }
   for (j=0; j<=num_data-1; j++) 
     if (zed[j]<=azed[0]) azedi = j;

/*
 *  one point upper and lower bounds
 */
   for (j=0; j<=azedi; j++)
     fb1[j] = MIN(aepkk[0],azed[0]*aepkk[0]/zed[j]);
   for (j=azedi+1; j<=num_data-1; j++)
     fb1[j] = MAX(aepkk[0],azed[0]*aepkk[0]/zed[j]);

/*
 *  check anchors against the one point bounds
 *    next anchor is at a larger frequency
 */
   if (num_anchors > 1){
     bcheck = interp1(zed,fb1,num_data,azed[1]-eps);
     if (azed[1] > azed[0])
       if (aepkk[1] < bcheck)
         derror("One point bound violated at second anchor");

/*
 *  check anchors against the one point bounds
 *    next anchor is at a smaller frequency
 */
     if (azed[1] < azed[0])
       if ( aepkk[1] > bcheck )
         derror("One point bound violated at second anchor");
   }
/*
 *  one point bound
 */
   if (num_anchors == 1){
     for (j=0; j<=azedi; j++){
       lo_bound[j] = fb1[j];
       up_bound[j] = kkreps[j];
     }
     for (j=azedi+1; j<=num_data-1; j++){
       up_bound[j] = kkreps[j];
       lo_bound[j] = fb1[j];
     }
   }

/*
 *  m-point bounds  (m>=2)
 *   loop over the number of bounds, 
 *   clear the previous bounds
 */
   z1[1] = 0; z1[2] = zed[0]; z1[3] = zed[num_data-1];
   z2[1] = 0; z2[2] = 0; z2[3] = zed[0];
   z3[1] = zed[0]; z3[2] = zed[num_data-1]; z3[3] = zed[num_data-1];

   for (m=2; m<=num_anchors; m++){

/*
 *  initialize for m-point bounds
 */
     kflag = 0;
     fQ = vector(1,m);
     b  = vector(1,m);
     indx  = ivector(1,m);
     A  = matrix(1,m,1,m);

/*
 *  loop over the number prescribed poles
 */
     for (k=0; k<=MIN(m,3); k++){

/*
 *  compute the number of free zeros and poles
 */
       alpha = (m+k-1)/2;
       beta  = (m-k)/2;

/*
 *  anchors to be used
 */
       for (j=1; j<=m; j++){
         anchzed[j] = azed[j-1];
         ancheps[j] = aepkk[j-1];
       }

/*
 *  loop over the possible prescribed poles
 */
       for (j=1; j<=npol[k]; j++){

/*
 *  prescribed poles
 */

         if (k==1) ppol[1] = z1[j];
         else if (k==2){
           ppol[1] = z2[j];
           ppol[2] = z3[j];
	 }
         else if (k==3){
           ppol[1] = z1[1];
           ppol[2] = z1[2];
           ppol[3] = z1[3];
	 }

/*
 *  Using a power basis for the numerator and
 *    denominator, interpolate the anchors.
 *    CAUTION: This is a Hilbert like matrix
 */

         for (jdat=1; jdat<=m; jdat++){
           Q = 1;
           for (kpol=1; kpol<=k; kpol++)
             Q = Q*(anchzed[jdat]-ppol[kpol]);
           fQ[jdat] = ancheps[jdat]*Q;
	 }
         
         for (jdat=1; jdat<=m; jdat++)
           for (n=1; n<=alpha+1; n++)
             A[jdat][n] = pow(anchzed[jdat],n-1);

         for (jdat=1; jdat<=m; jdat++)
           for (n=1; n<=beta; n++)
             A[jdat][n+alpha+1] = 
                -fQ[jdat]*pow(anchzed[jdat],n-1);

         for (jdat=1; jdat<=m; jdat++)
           b[jdat] = fQ[jdat]*pow(anchzed[jdat],beta);

/*
 *  Power basis coefficients of numerator and denominator
 */

         ludcmp(A,m,indx);
         lubksb(A,m,indx,b);

         for (jdat=0; jdat<=alpha; jdat++)
           nump[jdat] = b[jdat+1];

         denp[beta] = 1.0;
         for (jdat=0; jdat<=beta-1; jdat++)
           denp[jdat] = b[alpha+1+jdat+1];

/*
 *  compute and sort the poles and zeros
 */
         RRoots(nump, alpha, zer);
         RRoots(denp, beta,  pol);
         for (jpol=1; jpol<=k; jpol++) 
           pol[beta+jpol] = ppol[jpol];
         if ( k>0 ) piksrt(beta+k,pol);

/*
 *  interlace the poles and zeros -- pole first, then
 *    zero, then pole, etc...
 */
         for (jpol=1; jpol<=k+beta; jpol++)
           lace[2*jpol-1] = pol[jpol];
         for (jpol=1; jpol<=alpha; jpol++)
           lace[2*jpol] = zer[jpol];
         for (jpol=1; jpol<=k+beta+alpha; jpol++)
           slace[jpol] = lace[jpol];

/*
 *  check non-negativity of poles and zeros
 */
         fbm_ok = 0;
         if ((lmin = MIN(zer[1],pol[1])) < 0.0) 
            fbm_ok = fbm_ok + 1;

/*
 *  check for interlace of poles and zeros
 */
         lsrt = 0.0;
         piksrt(k+beta+alpha,slace);
         for (jpol=1; jpol<=k+beta+alpha; jpol++)
           lsrt = MAX(fabs(slace[jpol]-lace[jpol]),lsrt);
         if (lsrt > eps) fbm_ok = fbm_ok + 1;

/*
 *  check for all poles outside of (z_minus, z_plus)
 */
         for (jpol=1; jpol<=k+beta; jpol++)
           if ( pol[jpol] < z_plus )
             if (pol[jpol] > z_minus) fbm_ok = fbm_ok + 1;

/*
 *  if analytic constraints are satisfied, evaluate candidate 
 */
         if (fbm_ok == 0){
/*
 *  evaluate power basis polynomials just computed
 */
           for (jdat=0; jdat<=num_data-1; jdat++)
             numer[jdat] = polyval(nump,alpha,zed[jdat]);
           for (jdat=0; jdat<=num_data-1; jdat++)
             denom[jdat] = polyval(denp,beta,zed[jdat]);

/*
 *  evaluate prescribed polynomial in denominator
 */
           for (kpol=1; kpol<=k; kpol++){
             for (jdat=0; jdat<=num_data-1; jdat++)
               denom[jdat] = denom[jdat]*(zed[jdat]-ppol[kpol]);
	   }
/*
 *  calculate the candidate
 */
           for (jdat=0; jdat<=num_data-1; jdat++)
             fbm[jdat]  = (numer[jdat]/denom[jdat])+kkreps[jdat];
/*
 *  update the bound
 */
           if (kflag == 0){
             for (jdat=0; jdat<=num_data-1; jdat++){
               fb_lo[jdat] = fbm[jdat];
               fb_up[jdat] = fbm[jdat];
               kflag = 1;
	     }
	   }
           else if (kflag > 0){
             for (jdat=0; jdat<=num_data-1; jdat++){
               fb_lo[jdat] = MIN(fb_lo[jdat],fbm[jdat]);
               fb_lo[jdat] = MAX(fb_lo[jdat],-1./eps);
               fb_up[jdat] = MAX(fb_up[jdat],fbm[jdat]);
               fb_up[jdat] = MIN(fb_up[jdat],1./eps);
               kflag++;
	     }
	   }
	 }
/*
 *  close loop on possible prescribed poles (j)
 */
       }
/*
 *  close loop on number of prescribed poles (k)
 */
     }
/*
 *  evalulate the bounds if they have been computed
 */
     if (kflag >= 2){
       for (jdat=0; jdat<=num_data-1; jdat++){
         lo_bound[jdat] = fb_lo[jdat];
         up_bound[jdat] = fb_up[jdat];
       }
/*
 *  check for compatibitity at the next point
 */
       if (m < num_anchors){
         anchzc = azed[m];
         anchec = aepkk[m];
         lbchk = interp1(zed,fb_lo,num_data,anchzc);
         ubchk = interp1(zed,fb_up,num_data,anchzc);
         if ( anchec > lbchk )
           if ( anchec < ubchk )
             derror("Bounds violated at next anchor");
/*
 *  get ready to go again
 */
         free_vector(fQ,1,m);
         free_vector(b,1,m);
         free_ivector(indx,1,m);
         free_matrix(A,1,m,1,m);
       }
     }
/*
 *  bounds not computed, so outta here
 */
     if (kflag < 2) derror("All analytic constraints violated");
/*
 *  close loop on bound (m)
 */
   }

/*
 *  store the bounds
 */
   for (jdat=0; jdat<=num_data-1; jdat++){
     lo_bound[jdat] = fb_lo[jdat];
     up_bound[jdat] = fb_up[jdat];
   }

}

void GetInData (int argc, char** argv)
{
  int i,err,l;
  char line[MAXLINE];

/*  
 *  Parse the input arguments
 */
  for (i=1; i<=argc-1; i++){
    strcpy(line,argv[i]);
    if ( line[0] == '-' ){
      switch (line[1]){
        case 'h':
          err = 1;
          break;
        default:
          err = 0;
      }
    }
  }

  if (err == 1) {
    fprintf(stderr,"\n");
    fprintf(stderr,"Synposis: Computes the finite frequency Kramer-Kronig\n");
    fprintf(stderr,"  bounds.  See Milton, Eyre, and Mantese for details.\n");
    fprintf(stderr,"\n");
    fprintf(stderr,"Usage: kkbound [-h] < infile > outfile \n");
    fprintf(stderr,"Required argument:\n");
    fprintf(stderr,"  infile -- input file name\n");
    fprintf(stderr,"\n");
    fprintf(stderr,"Optional arguments:\n");
    fprintf(stderr,"  -h  -- this message\n");
    fprintf(stderr,"  outfile -- output file name\n");
    fprintf(stderr,"\n");
    fprintf(stderr,"The input file must have the following data:\n\n");
    fprintf(stderr,"  n = number of anchors\n");
    fprintf(stderr,"  w_1, w_2, ..., w_n = frequencies of the anchors\n");
    fprintf(stderr,"  r_1, r_2, ..., r_n = real dielectric data");
    fprintf(stderr," at the anchors\n\n");
    fprintf(stderr,"  m = number of data points\n");
    fprintf(stderr,"  W_1, W_2, ..., W_m = frequencies of the data\n");
    fprintf(stderr,"  i_1, i_2, ..., i_m = imaginary dielectric data\n\n");
    fprintf(stderr,"  The W_i must be ordered, i.e. \n");
    fprintf(stderr,"      W_1 <  W_2 < W_3 < ... < W_m \n\n");
    fprintf(stderr,"The input file structure:\n\n");
    fprintf(stderr,"  n\n");
    fprintf(stderr,"  w_1  r_1 \n");
    fprintf(stderr,"  w_2  r_2 \n");
    fprintf(stderr,"  ...  ... \n");
    fprintf(stderr,"  w_n  r_n \n");
    fprintf(stderr,"  m\n");
    fprintf(stderr,"  W_1  i_1 \n");
    fprintf(stderr,"  W_2  i_2 \n");
    fprintf(stderr,"  ...  ...     \n");
    fprintf(stderr,"  W_m  i_m \n\n");
    fprintf(stderr," n and m are integers\n");
    fprintf(stderr," the remaining data are floating point\n\n");
    fprintf(stderr,"The output is to stdout:\n\n");
    fprintf(stderr,"  W_1, W_2, ..., W_m = frequencies of the data\n");
    fprintf(stderr,"  L_1, L_2, ..., L_m = lower bound for the data\n");
    fprintf(stderr,"  U_1, U_2, ..., U_m = upper bound for the data\n\n");
    fprintf(stderr,"The output format is:\n\n");
    fprintf(stderr,"  W_1  L_1  U_1\n");
    fprintf(stderr,"  W_2  L_2  U_2\n");
    fprintf(stderr,"  ...  ...  ...   \n");
    fprintf(stderr,"  W_m  L_m  U_m\n\n");
    exit(0);
  }
}
void main(int argc, char** argv) {

   int num_data, num_anchors, j,k;

   float *Omega,*ieps;
   float *aOmega,*areps;
   float *up_bound,*lo_bound;

   GetInData (argc,argv);

   j=fscanf(stdin,"%i",&num_anchors);
   if ( j!= 1 )
     derror("Data error: First entry must be the number of anchors");
   aOmega = vector(0,num_anchors);
   areps = vector(0,num_anchors);
   for (k=0;k<=num_anchors-1;k++){
     j=fscanf(stdin,"%e %e",&aOmega[k],&areps[k]);
     if ( j!= 2 )
       derror("Data error: The anchors and real eps are incorrect");
   }

   j=fscanf(stdin,"%i",&num_data);
   if ( j!= 1 )
     derror("Data error: The number of data points follows the anchors");
   Omega = vector(0,num_data);
   ieps = vector(0,num_data);
   for (k=0;k<=num_data-1;k++){
     j=fscanf(stdin,"%e %e",&Omega[k],&ieps[k]);
     if ( j!= 2 )
       derror("Data error: The frequencies/data are incorrect");
   }

   up_bound = vector(0,num_data);
   lo_bound = vector(0,num_data);

   bnds(Omega,ieps,num_data,aOmega,areps,num_anchors,
           up_bound, lo_bound);

   for (j=0; j<=num_data-1; j++)
      printf("%e %e %e\n",Omega[j],lo_bound[j],up_bound[j]);

   exit(0);

}
